package eShopper;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.sql.*;
import javax.help.*;

/*
 * Created on Apr 27, 2004
 *
 * Made to seperate the GUI from the program logic, at least a little bit
 * Also made so that subclasses of eShopper won't get their ActionPerformed methods
 * all confused (like closing a pop-up window closes the whole program).
 * 
 * TODO: This has gone through a lot of revisions, so it needs cleaned out.
 * 
 */

/**
 * This is the main eShopper window.  This window appears when the program
 * is started, and the other windows are created from it.
 * 
 * @author Jim Laing
 *
 */
public class MainWindow extends eShopperGUI {
	
	// We need a clipboard for copy and paste actions
	eShopperClipboard clipboard;
	
	// Undo DEBUG
	//eShopperAction anAction = new eShopperAction();
	
	JLabel timeLabel; // label for the current time period
	// List Tab
	JPanel listPanel = new JPanel(); // attached to list tab
	JPanel listButtons = new JPanel(); // contains list buttons
	JPanel listLeftPanel = new JPanel(); // contains the stuff on the left of the list tab
	JPanel listViewPanel = new JPanel(); // Contains view buttons
	JScrollPane listView; // The pane to hold the list
	
	// Menus	
	JMenuBar menuBar = new JMenuBar(); // menu bar
	JMenu fileMenu, listSubMenu, editMenu, viewMenu, helpMenu, toolsMenu; // menu item
	JMenuItem menuItem = new JMenuItem(); // generic menu entry thing
	
	// Some buttons need actions performed on them
	JButton recipeViewButton, dateViewButton, itemViewButton = new JButton();
	
	// Help Stuff
	HelpSet hs;
	HelpBroker hb;
		
	public MainWindow(eShopperList list) {
		this.list = list;
		this.database = list.getDB();
		
		// Find the HelpSet file and create the HelpSet object:
		String helpHS = "Help/eShopper.hs";
		ClassLoader cl = eShopper.class.getClassLoader();
		try {
			java.net.URL hsURL = HelpSet.findHelpSet(cl, helpHS);
			hs = new HelpSet(null, hsURL);
		} catch (Exception ee) {
			// Say what the exception really is
			System.out.println( "HelpSet " + ee.getMessage());
			System.out.println("HelpSet "+ helpHS +" not found");
			return;
		}
		
		// Create a HelpBroker object:
		hb = hs.createHelpBroker();
		
		clipboard = new eShopperClipboard(list.getDB());
		
		// make list panel
		listPanel.setName("listPanel");
		listPanel.setLayout(new GridLayout(1,1));
	
		listPanel.setLayout(new BorderLayout());
		listPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		listLeftPanel.setLayout( new BoxLayout(listLeftPanel, BoxLayout.Y_AXIS) );
	
		// Add the View Panel
		listViewPanel.setLayout( new BoxLayout(listViewPanel, BoxLayout.X_AXIS));
		listViewPanel.add(new JLabel("Organize by:  "));
	
		// Let's use the same JButton for all of our buttons!
		JButton aButton = new JButton();
	
		// Recipe View Button
		//recipeViewButton = new JButton("Recipe");
		recipeViewButton = new JButton("Recipe", createImageIcon("images/recipeViewIcon.gif"));
		recipeViewButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		recipeViewButton.setHorizontalTextPosition(AbstractButton.CENTER);
		recipeViewButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		recipeViewButton.setMinimumSize( new Dimension( 90, 80) );
		recipeViewButton.setMaximumSize( new Dimension( 90, 80) );
		recipeViewButton.setPreferredSize( new Dimension( 90, 80) );
		recipeViewButton.setMnemonic(KeyEvent.VK_R);
		recipeViewButton.addActionListener(this);
		recipeViewButton.setActionCommand("setRecipeView");
		listViewPanel.add(recipeViewButton);
	
		// Date View Button
		listViewPanel.add( Box.createRigidArea(new Dimension(10,0)) );
		//dateViewButton = new JButton("Date");
		dateViewButton = new JButton("Date", createImageIcon("images/dateViewIcon.gif"));
		dateViewButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		dateViewButton.setHorizontalTextPosition(AbstractButton.CENTER);
		dateViewButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		dateViewButton.setMinimumSize( new Dimension( 90, 80) );
		dateViewButton.setMaximumSize( new Dimension( 90, 80) );
		dateViewButton.setPreferredSize( new Dimension( 90, 80) );
		dateViewButton.setMnemonic(KeyEvent.VK_D);
		dateViewButton.addActionListener(this);
		dateViewButton.setActionCommand("setDateView");
		dateViewButton.setEnabled(false);
		listViewPanel.add(dateViewButton);
		
		// Item View Button
		listViewPanel.add( Box.createRigidArea(new Dimension(10,0)) );
		//itemViewButton = new JButton("Item");
		itemViewButton = new JButton("Item", createImageIcon("images/itemViewIcon.gif"));
		itemViewButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		itemViewButton.setHorizontalTextPosition(AbstractButton.CENTER);
		itemViewButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		itemViewButton.setMinimumSize( new Dimension( 90, 80) );
		itemViewButton.setMaximumSize( new Dimension( 90, 80) );
		itemViewButton.setPreferredSize( new Dimension( 90, 80) );
		itemViewButton.setMnemonic(KeyEvent.VK_I);
		itemViewButton.addActionListener(this);
		itemViewButton.setActionCommand("setItemView");
		listViewPanel.add(itemViewButton);
	
		// Time Span label
		listViewPanel.add(Box.createHorizontalGlue());
		//listViewPanel.add(new JLabel(" Days In List:  March 16 - March 23"));
	
		listViewPanel.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
		listPanel.add(listViewPanel, BorderLayout.NORTH);
		
		// create shopping list
		listView = new JScrollPane(list);
		listPanel.add(listView, BorderLayout.CENTER);
		listButtons.setLayout( new BoxLayout(listButtons, BoxLayout.Y_AXIS) );
	
		// Add the "Add Item" Button
		aButton = new JButton("Add Item");
		aButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		aButton.setMinimumSize( new Dimension( 135, 25) );
		aButton.setMaximumSize( new Dimension( 135, 25) );
		aButton.setPreferredSize( new Dimension( 135, 25) );
		aButton.setMnemonic(KeyEvent.VK_A);
		aButton.addActionListener(this);
		aButton.setActionCommand("add");
		listButtons.add(aButton);
	
	    // Add the "Remove Item(s)" Button
		listButtons.add( Box.createRigidArea(new Dimension(0,5)) );
		aButton = new JButton("Remove Item(s)");
		aButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		aButton.setMinimumSize( new Dimension( 135, 25) );
		aButton.setMaximumSize( new Dimension( 135, 25) );
		aButton.setPreferredSize( new Dimension( 135, 25) );
		aButton.setMnemonic(KeyEvent.VK_M);
		aButton.addActionListener(this);
		aButton.setActionCommand("remove");
		listButtons.add(aButton);
	
		// Add the "Clear List" Button
		listButtons.add( Box.createRigidArea(new Dimension(0,5)) );
		aButton = new JButton("Clear List");
		aButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		aButton.setMinimumSize( new Dimension( 135, 25) );
		aButton.setMaximumSize( new Dimension( 135, 25) );
		aButton.setPreferredSize( new Dimension( 135, 25) );
		aButton.setMnemonic(KeyEvent.VK_L);
		aButton.addActionListener(this);
		aButton.setActionCommand("clear");
		listButtons.add(aButton);
		
		// Send to Palm button
		listButtons.add( Box.createRigidArea(new Dimension(0,5)) );
		aButton = new JButton("Send to PDA");
		aButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		aButton.setMinimumSize( new Dimension( 135, 25) );
		aButton.setMaximumSize( new Dimension( 135, 25) );
		aButton.setPreferredSize( new Dimension( 135, 25) );
		aButton.setMnemonic(KeyEvent.VK_S);
		aButton.addActionListener(this);
		aButton.setActionCommand("export");
		listButtons.add(aButton);
	
		// Put a border around all of the buttons
		listButtons.setBorder(BorderFactory.createEmptyBorder(0,10,10,0));
		// Add buttons to the window
		listPanel.add(listButtons, BorderLayout.EAST);
	

		getContentPane().add(listPanel);
		
		// make menu bar
		setJMenuBar(menuBar);
		
	    // file menu
		fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		listSubMenu = new JMenu("List");
		listSubMenu.setMnemonic(KeyEvent.VK_L);
		createListMenu();
		fileMenu.add(listSubMenu);
		menuItem = new JMenuItem("Close");
		menuItem.setMnemonic(KeyEvent.VK_C);
		menuItem.getAccessibleContext().setAccessibleDescription(
		"Close eShopper.");
		menuItem.addActionListener(this);
		menuItem.setActionCommand("exit");
		fileMenu.add(menuItem);
		menuBar.add(fileMenu);
	
		// edit menu
		editMenu = new JMenu("Edit");
		editMenu.setMnemonic(KeyEvent.VK_E);
		menuItem = new JMenuItem("Undo");
		menuItem.addActionListener(this);
		menuItem.setActionCommand("undo");
		menuItem.setMnemonic(KeyEvent.VK_U);
		editMenu.add(menuItem);
		menuItem = new JMenuItem("Redo");
		menuItem.addActionListener(this);
		menuItem.setActionCommand("redo");
		menuItem.setMnemonic(KeyEvent.VK_R);
		editMenu.add(menuItem);
		editMenu.addSeparator();
		menuItem = new JMenuItem("Cut");
		menuItem.addActionListener(this);
		menuItem.setActionCommand("cut");
		menuItem.setMnemonic(KeyEvent.VK_T);
		editMenu.add(menuItem);
		menuItem = new JMenuItem("Copy");
		menuItem.setMnemonic(KeyEvent.VK_C);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("copy");
		editMenu.add(menuItem);
		menuItem = new JMenuItem("Paste");
		menuItem.addActionListener(this);
		menuItem.setMnemonic(KeyEvent.VK_P);
		menuItem.setActionCommand("paste");
		editMenu.add(menuItem);
		menuBar.add(editMenu);
	
		// view menu
		viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);
		menuItem = new JMenuItem("Recipe");
		menuItem.setMnemonic(KeyEvent.VK_R);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("setRecipeView");
		viewMenu.add(menuItem);
		menuItem = new JMenuItem("Date");
		menuItem.setMnemonic(KeyEvent.VK_D);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("setDateView");
		viewMenu.add(menuItem);
		menuItem = new JMenuItem("Item");
		menuItem.setMnemonic(KeyEvent.VK_I);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("setItemView");
		viewMenu.add(menuItem);
		menuBar.add(viewMenu);
	
		// tools menu
		toolsMenu = new JMenu("Tools");
		toolsMenu.setMnemonic(KeyEvent.VK_T);
		menuItem = new JMenuItem("Manage Recipes...");
		menuItem.setMnemonic(KeyEvent.VK_R);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("Manage Recipes");
		toolsMenu.add(menuItem);
		menuItem = new JMenuItem("Manage Ingredients...");
		menuItem.setMnemonic(KeyEvent.VK_I);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("Manage Ingredients");
		toolsMenu.add(menuItem);
		menuItem = new JMenuItem("Send to PDA");
		menuItem.setMnemonic(KeyEvent.VK_S);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("export");
		toolsMenu.add(menuItem);
		menuBar.add(toolsMenu);
	
		// help menu
		helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		menuItem = new JMenuItem("Contents");
		menuItem.addActionListener(this);
		//menuItem.setActionCommand("Help Contents");
		menuItem.addActionListener(new CSH.DisplayHelpFromSource( hb ));
		menuItem.setMnemonic(KeyEvent.VK_C);
		helpMenu.add(menuItem);
		//menuItem = new JMenuItem("About");
		//menuItem.setMnemonic(KeyEvent.VK_A);
		//helpMenu.add(menuItem);
		menuBar.add(helpMenu);

		// exit program on window close	
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
			System.exit(0);
		 }});
	}

	/**
	 * This generates the File -> List menu.  It gets the current lists
	 * from the database, and displays them for the user to choose one.  It
	 * also has the button for making a new list.
	 *
	 */
	public void createListMenu(){
		menuItem = new JMenuItem("New");
		menuItem.setMnemonic(KeyEvent.VK_N);
		menuItem.addActionListener(this);
		menuItem.setActionCommand("New List");
		listSubMenu.add(menuItem);
		listSubMenu.addSeparator();
		try{
			ResultSet rs = database.query("SELECT ID, Name FROM List WHERE ID > -1;");
			while (rs.next()){
				menuItem = new JMenuItem(rs.getString(2));
				menuItem.addActionListener(this);
				menuItem.setActionCommand("Change lists");
				listSubMenu.add(menuItem);
			}
			rs.close();
		} catch (SQLException se){
			System.out.println("Could not get lists from database.  Error:");
			se.printStackTrace();
		}
	}

	/**
	 * Creates a new List.  The user is prompted for a name for the list.
	 *
	 */
	public void createNewList(){
		try{
	
			//get new list ID
			ResultSet rs = database.query("SELECT TOP 1 ID FROM List ORDER BY ID DESC;");
			rs.next();
			Integer index = (new Integer(rs.getInt(1) + 1));
			rs.close();
			
			//get list name
			String listName;
			listName = JOptionPane.showInputDialog(null, "Enter a name for your list:",
				"UntitledList" + index.toString());
			if (listName != null){
				listName = listName.replaceAll("'", "''");
				System.out.println(listName);
			
				// Check to see if there is already a list with that name
				rs = database.query("SELECT Name FROM List WHERE Name = '" + listName + "'");
				if (rs.next()){
					JOptionPane.showMessageDialog(listPanel,
	    				    "Error: List " + rs.getString(1) + " already exists.",
	    				    "eShopper Error",
	    				    JOptionPane.ERROR_MESSAGE);
	    			return;
				}
				
				//update database
				database.update("INSERT INTO List (ID, Name) VALUES ("
						+ index.toString() + ", '" + listName + "');");
				//change active list to the new list
				list.setListID(index.intValue());
				//update list menu
				menuItem = new JMenuItem(listName);
				menuItem.addActionListener(this);
				menuItem.setActionCommand("Change lists");
				listSubMenu.add(menuItem);
			}
		} catch (SQLException se){
			System.out.println("Could not create new list.  Error:");
			se.printStackTrace();
		}
	}

	/**
	 * This loads an existing list from the database into the main window.
	 * 
	 * @param listName The name of the list to be loaded.
	 */
	public void loadList(String listName){
		try{
			ResultSet rs = database.query("SELECT ID FROM List WHERE Name = '" + listName + "';");
			rs.next();
			list.setListID(rs.getInt(1));
		}
		catch (SQLException se){
			System.out.println("Could not create access list.  Error:");
			se.printStackTrace();
		}
	}
	
	
	/**
	 * This handles all of the ActionEvents generated by menu.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("exit")) {
			System.exit(0);
		} else if (e.getActionCommand().equals("New List"))
			createNewList();
		else if (e.getActionCommand().equals("Change lists"))
			loadList( ((JMenuItem)e.getSource()).getText());
		else if (e.getActionCommand().equals("undo")){
			// anAction.undoAction(list); // Undo DEBUG 
		} else if (e.getActionCommand().equals("redo")){
			
		} else if (e.getActionCommand().equals("cut")){
			if (list.getSelectionCount() > 0)
				clipboard.cut(list);
		} else if (e.getActionCommand().equals("copy")){
			if (list.getSelectionCount() > 0) 
				clipboard.add((eShopperNode) list.getSelectionPath().getLastPathComponent());
		} else if (e.getActionCommand().equals("paste"))
			clipboard.paste(list);
		else if (e.getActionCommand().equals("add"))
			displayAddItemWindow();
		else if (e.getActionCommand().equals("clear")){
			int n = JOptionPane.showConfirmDialog(
	                listPanel, "Are you sure you want to clear the list?",
	                "Clear Confirmation",
	                JOptionPane.YES_NO_OPTION);
	        if (n == JOptionPane.YES_OPTION) {
	        	// anAction.setAction(1, list); // Undo DEBUG 
	        	list.clear();
	        }
		} else if (e.getActionCommand().equals("remove")){
			// Prompt the user to make sure 
			int n = JOptionPane.showConfirmDialog(
	                listPanel, "Are you sure you want to remove this?",
	                "Remove Confirmation",
	                JOptionPane.YES_NO_OPTION);
	        if (n == JOptionPane.YES_OPTION) {
	        	list.remove();
	        }
		}else if (e.getActionCommand().equals("Manage Recipes"))
			displayManageRecipesWindow();
		else if (e.getActionCommand().equals("Manage Ingredients"))
			displayManageIngredientsWindow();
		else if (e.getActionCommand().equals("setRecipeView")){
			list.setView(eShopperRoot.RECIPE);
			recipeViewButton.setEnabled(false);
			dateViewButton.setEnabled(true);
			itemViewButton.setEnabled(true);
		} else if (e.getActionCommand().equals("setDateView")){
			list.setView(eShopperRoot.DATE);
			recipeViewButton.setEnabled(true);
			dateViewButton.setEnabled(false);
			itemViewButton.setEnabled(true);
		} else if (e.getActionCommand().equals("setItemView")){
			list.setView(eShopperRoot.ITEM);
			recipeViewButton.setEnabled(true);
			dateViewButton.setEnabled(true);
			itemViewButton.setEnabled(false);
		} else if (e.getActionCommand().equals("export")){
			int n = JOptionPane.showConfirmDialog(
	                listPanel, "Are you sure you want to send the list?",
	                "Send Confirmation",
	                JOptionPane.YES_NO_OPTION);
	        if (n == JOptionPane.YES_OPTION) {
	        	new handyShopperDB(list);
	        }
		}
	}
	

}
