package eShopper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.sql.*;

/**
 * This window allows you to edit pre-existing recipes in the database or add
 * a new recipe thereto.  It differs from a RecipeWindow in that it contains a JComboBox of
 * all of the recipes in the database and a button to add a new one.
 * 
 * @author Nathan
 */
public class ManageRecipesWindow extends RecipeWindow {
	
	ManageRecipesWindow(eShopperList list){
		super(list);
		this.list = list;
		this.database = list.getDB();
		
		setTitle("eShopper: Manage Recipes");
		setSize(400, 525);
		
		//
		// edit namePanel
		//
		
		namePanel.setMinimumSize( new Dimension(320, 65) );
		namePanel.setMaximumSize( new Dimension(320, 65) );
		namePanel.setPreferredSize( new Dimension(320, 65) );
		namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.Y_AXIS));
		JPanel nameSubPanel = new JPanel();
		nameSubPanel.add(new JLabel("Select a recipe: "));
		
		//populate recipesBox from database
		Vector recipeVector = new Vector();
		try{
			ResultSet rs = database.query("SELECT ID, Name FROM Recipe;");
			while (rs.next()){
				recipeVector.add(new Recipe(rs.getInt(1), rs.getString(2)));
			}
			rs.close();
		}
		catch (SQLException se) {
			System.out.println("Unable to get recipes from database. Error:");
			se.printStackTrace();
		}
		recipesBox = new JComboBox(recipeVector);
		recipesBox.addActionListener(this);
		recipesBox.setActionCommand("recipesBox");
		nameSubPanel.add(recipesBox);
		namePanel.add(nameSubPanel);
		
		// The buttons
		JPanel buttonSubPanel = new JPanel();
		buttonSubPanel.setLayout(new BoxLayout(buttonSubPanel, BoxLayout.X_AXIS));
		buttonSubPanel.setMinimumSize( new Dimension(320, 25) );
		buttonSubPanel.setMaximumSize( new Dimension(320, 25) );
		buttonSubPanel.setPreferredSize( new Dimension(320, 25) );
		buttonSubPanel.add(Box.createHorizontalGlue());
		
		// Delete button
		JButton deleteIngredientButton = new JButton("Delete");
		deleteIngredientButton.addActionListener(this);
		deleteIngredientButton.setActionCommand("delete");
		deleteIngredientButton.setMnemonic(KeyEvent.VK_D);
		buttonSubPanel.add(deleteIngredientButton);
		buttonSubPanel.add( Box.createRigidArea(new Dimension(5,0)) );
				
		// Rename button
		JButton renameIngredientButton = new JButton("Rename");
		renameIngredientButton.addActionListener(this);
		renameIngredientButton.setActionCommand("rename");
		renameIngredientButton.setMnemonic(KeyEvent.VK_R);
		buttonSubPanel.add(renameIngredientButton);
		buttonSubPanel.add( Box.createRigidArea(new Dimension(5,0)) );
		
		// Create button
		JButton addRecipeButton = new JButton("Create Recipe...");
		addRecipeButton.addActionListener(this);
		addRecipeButton.setActionCommand("Create Recipe");
		buttonSubPanel.add(addRecipeButton);
		namePanel.add( buttonSubPanel );
		
		// fill ingredients from database
		getIngredients((Recipe)recipesBox.getSelectedItem());
		
		mainPanel.add(namePanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(ingredientsPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(descriptionPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(instructionsPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,17)) );
		mainPanel.add(buttonPanel);
		getContentPane().add(mainPanel);
		
		getDescription((Recipe)recipesBox.getSelectedItem());
		getInstructions((Recipe)recipesBox.getSelectedItem());
	}
	
	/**
	 * This gets all of the ingredients in a recipe, and adds them
	 * to the ingredientVector that the pulldown menu is built from.
	 * 
	 * @param recipe The recipe whose ingredients should be retrieved.
	 */
	public void getIngredients(Recipe recipe){
		ingredientVector.removeAllElements();
		try{
			String q = new String();
			q = "SELECT Ingredient.ID, Ingredient.Name, Has.Amount, Has.UnitID " +
				"FROM Ingredient, Has " + 
				"WHERE Ingredient.ID = Has.IngredientID AND " +
				"Has.RecipeID = " + recipe.getID();
			ResultSet rs = database.query(q);
			while(rs.next()){
				int ingredientID = rs.getInt(1);
				String ingredientName = rs.getString(2);
				double amount = rs.getDouble(3);
				eShopperUnit unit = new eShopperUnit(rs.getInt(4), database);
				ingredientVector.add(new RecipeIngredient(ingredientID, ingredientName, amount, unit));
			}
			rs.close();
			ingredientList.setListData(ingredientVector);
		}
		catch (SQLException se) {
			System.out.println("Unable to get ingredients from database. Error:");
			se.printStackTrace();
		}
	}
	
	/** 
	 * This fills the description text area on the window with
	 * the description for the given recipe.
	 * 
	 * @param recipe The recipe whose description should be retrieved.
	 */
	public void getDescription(Recipe recipe){
		try{
			ResultSet rs = database.query("SELECT Description FROM Recipe WHERE ID = " + recipe.getID());
			rs.next();
			descArea.setText(rs.getString(1));
			rs.close();
		}
		catch (SQLException se) {
			System.out.println("Unable to get description from database. Error:");
			se.printStackTrace();
			descArea.setText("");
		}
	}
	
	/**
	 * This fills the instructions text area on the window with the
	 * instructions for the given recipe.
	 * 
	 * @param recipe The recipe whose instructions should be retrieved.
	 */
	public void getInstructions(Recipe recipe){
		try{
			ResultSet rs = database.query("SELECT Directions FROM Recipe WHERE ID = " + recipe.getID());
			rs.next();
			instructionsArea.setText(rs.getString(1));
			rs.close();
		}
		catch (SQLException se) {
			System.out.println("Unable to get directions from database. Error:");
			se.printStackTrace();
			descArea.setText("");
		}
	}

	/**
	 * This saves the information appearing on the screen to the 
	 * database.
	 */
	public void updateDatabase(){
		Recipe recipe = (Recipe)recipesBox.getSelectedItem();
		Vector oldIngredients = new Vector();
		try{
			// get ingredient list before changes
			String q = new String();
			q = "SELECT Ingredient.ID, Ingredient.Name, Has.Amount, Unit.ID " +
				"FROM Ingredient, Recipe, Has, Unit WHERE Ingredient.ID = " +
				"Has.IngredientID AND HAS.UNITID = UNIT.ID AND " +
				"Has.RecipeID = " + recipe.getID();
			ResultSet rs = database.query(q);
			while(rs.next()){
				int ingredientID = rs.getInt(1);
				String ingredientName = rs.getString(2);
				double amount = rs.getDouble(3);
				eShopperUnit unit = new eShopperUnit(rs.getInt(4), database);
				oldIngredients.add(new RecipeIngredient(ingredientID, ingredientName, amount, unit));
			}
			rs.close();
			
			// remove old ingredients from recipe
			for (int i = 0; i < oldIngredients.size(); i++){
				RecipeIngredient ingredient = (RecipeIngredient)oldIngredients.elementAt(i);
				if (!ingredientVector.contains(ingredient)){
				
					//remove the ingredient
					database.update("DELETE FROM Has WHERE RecipeID = " + recipe.getID() 
						+ " AND IngredientID = " + ingredient.getID() + ";");
				}
			}
			
			// add new ingredients to recipe
			for (int i = 0; i < ingredientVector.size(); i++){
				RecipeIngredient ingredient = (RecipeIngredient)ingredientVector.elementAt(i);
				if (!oldIngredients.contains(ingredient)){

					//add the ingredient
					database.update("INSERT INTO Has (RecipeID, IngredientID, Amount, UnitID) VALUES ("
						+ recipe.getID() + ", " + ingredient.getID() + ", " + ingredient.getAmount() + ", "
					+ ingredient.getUnit().getID() + ");");
				}
			}
			
			//update Description and Directions
			database.update("UPDATE Recipe SET Description = '" + descArea.getText().replaceAll("'", "''")
				+ "', Directions = '" + instructionsArea.getText().replaceAll("'", "''")
				+ "' WHERE ID = '" + recipe.getID() + "';");
			JOptionPane.showMessageDialog(this, "Database updated");
		}
		catch (SQLException se) {
			System.out.println("Unable to update database. Error:");
			se.printStackTrace();
		}
	}
	
	/**
	 * This renames the recipe currently selected in the combobox.  A dialog
	 * window pops up that prompts the user for the new name.  The recipe's name
	 * is then updated both in the database and in the combobox.
	 */
	public void renameRecipe(){
		String inputValue = JOptionPane.showInputDialog("Enter the new name for " + recipesBox.getSelectedItem());
		 if (inputValue != null){
		 	try{
		 		
		 		// The database doesn't like apostrophes.
				String cleanName = inputValue;
				cleanName = cleanName.replaceAll("'", "''");
				
				// First check to see if there is already a recipe with the
				// same name.
				String q = "SELECT Name FROM Recipe where Name = '" + cleanName + "'";
				ResultSet rs = database.query(q);
				if (rs.next()){
					JOptionPane.showMessageDialog(mainPanel,
    				    "Error: Recipe " + rs.getString(1) + " already exists.\n" +
						"Please choose another name.",
    				    "eShopper Error",
    				    JOptionPane.ERROR_MESSAGE);
					return;
				}
		 	
				// Next, rename the recipe in the database.
				q = "UPDATE Recipe SET Name = '" + cleanName + "' WHERE ID = '" + ((Recipe)recipesBox.getSelectedItem()).getID() + "'";
				database.update(q);
		 	}
		 	catch (SQLException se) {
		 		System.out.println("Error: ManageRecipesWindow::renameRecipe");
		 		se.printStackTrace();
		 	}
		 			 	
		 	// Then, rename in the comboBox by deleting the old
		 	// name, then inserting the new one.
		 	int recipeID = ((Recipe)recipesBox.getSelectedItem()).getID();
		 	int n = recipesBox.getSelectedIndex();
		 	recipesBox.removeItemAt(n);
		 	recipesBox.insertItemAt(new Recipe(recipeID, inputValue), n);
		 	recipesBox.setSelectedIndex(n);
		 	list.update();
		 }
	}
	
	/**
	 * This removes the currently selected recipe from the recipe database
	 * and all of the shopping lists.  It prompts the user for confirmation
	 * before doing so.
	 *
	 */
	public void deleteRecipe(){
		Recipe recipe = (Recipe)recipesBox.getSelectedItem();
		int n = JOptionPane.showConfirmDialog(
                mainPanel, "Are you sure you want to delete recipe " + recipe + "?\n" +
				"This will also delete it from all shopping lists.",
                "Delete Confirmation",
                JOptionPane.YES_NO_OPTION);
        if (n == JOptionPane.YES_OPTION) {
        	// If yes, delete from the database.
            try{
               	// Delete from all shopping lists.
               	String q = "DELETE FROM RecipeList WHERE RecipeID = '" + recipe.getID() + "'";
               	database.update(q);
               	
               	// Delete from Ingredients database.
               	q = "DELETE FROM Recipe WHERE ID = '" + recipe.getID() + "'";
               	database.update(q);
            }
            catch (SQLException se) {
    			System.out.println("Error: ManageRecipesWindow::deleteRecipe");
    			se.printStackTrace();
    		}
            
            // Then, delete from the combobox.
            recipesBox.removeItem(recipe);
            
            list.update();
        }
	}
	
	/**
	 * This handles all of the ActionEvents generated by the buttons in the window.
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		if (e.getActionCommand().equals("Create Recipe"))
			displayCreateRecipeWindow();
		else if (e.getActionCommand().equals("recipesBox")){
			getIngredients((Recipe)recipesBox.getSelectedItem());
			getDescription((Recipe)recipesBox.getSelectedItem());
			getInstructions((Recipe)recipesBox.getSelectedItem());
		} else if (e.getActionCommand().equals("rename"))
			renameRecipe();	
		else if (e.getActionCommand().equals("delete"))
			deleteRecipe();
	}
}
