package eShopper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;


/*
 * Created on May 12, 2004
 *
 */
 

/**
 * Takes the list and creates a vector of ingredients which can be
 * used both by the item view and the export functions.
 * 
 * This class goes out to Keith Vander Linden!
 * He's the coolest guy at Calvin College.  
 * And me? I'm not in college anymore!! 
 * (i really should've joined the coast guard ... uh, on with the class!)
 * 
 * @author Jim Laing
 *
 */
public class eShopperCombiner {

	private Vector ingredientVector;
	private eShopperDB db;
	private int listID;
	private boolean largest;

	/**
	 * Takes a shopping list and creates a vector of all the ingredients on the list
	 * where similar ingredients have been combined according to units. Useful in
	 * creating both the "Item" view and the export list
	 * @param listID The shopping list from which to get the ingredients
	 * @param db The database holding the eShopper database.
	 * @param largestUnit A boolean which expresses whether to use the
	 * largest or smallest unit of all the units for a ingredient which were passed.
	 * True for largest, otherwise smallest.
	 */
	public eShopperCombiner(int listID, eShopperDB db, boolean largestUnit) {
		this.db = db;
		this.listID = listID;
		this.largest = largestUnit;
		ingredientVector = new Vector();
	}

	/**
	 * @return A vector of ingredients, properly combined according to units
	 * and expressed in either the largest or smallest original unit
	 */
	public Vector getIngredientList() {
		return ingredientVector;
	}

	/**
	 * @param largest A boolean. True if the largest unit passed is desire. False if
	 * the smallest is desired.
	 */
	public void setLargestUnit(boolean largest) {
		this.largest = largest;
	}
	
	/**
	 * 
	 * @return a vector of the ingredients currently on the shopping list,
	 * combined to the greatest degree possible, according to the value of the
	 * largest boolean
	 */
	public Vector getCombinedIngredients() {
		Vector v = new Vector();
		String q = new String();
		Vector aIngrd = new Vector();
	
		try{
			// get all the ingredients on the list, both on the INGREDIENTLIST 
			// table and the HAS table for recipes on the RECIPELIST table
			// in an overall ordered way
			q = "SELECT * FROM ( " +  
				"SELECT INGREDIENTID, AMOUNT, UNITID FROM " +
				"INGREDIENTLIST WHERE LISTID = " + listID + " UNION ALL " +
				"SELECT INGREDIENTID, AMOUNT, UNITID FROM " +
				"RECIPE, HAS, INGREDIENT, RECIPELIST WHERE INGREDIENT.ID = " +
				"HAS.INGREDIENTID AND HAS.RECIPEID = RECIPE.ID AND RECIPE.ID = " +
				"RECIPELIST.RECIPEID AND RECIPELIST.LISTID = " + listID + 
				") ORDER BY INGREDIENTID ";
			ResultSet rs = db.query(q);
			if(rs.next()) {
			// this traverses the resultset: things in the outer loop get performed 
			// on the first ingredient of a new type while things in the inner 
			// loop get performed on the set of similar ingredients, if it exists
				out: while(true) {
					int curID = rs.getInt("INGREDIENTID");
					for(int i = 0; rs.getInt("INGREDIENTID") == curID; i++) {
						// put all the ingredient of the same type in one vector
						aIngrd.add(new eShopperIngredient(0, curID, listID, 
							db, rs.getFloat("AMOUNT"), rs.getInt("UNITID")));
						if(!rs.next()) break out;	
					}
					v.addAll(combine(aIngrd));
					aIngrd.clear();
				}
			}
			v.addAll(combine(aIngrd));
			return v;
		} catch (SQLException e) {
			System.out.println("Error: eShopperCombiner::getCombinedIngredients");
			e.printStackTrace();	
		}					
		return null;
	}
	
	/**
	 * Combines ingredients of the same type
	 * @param aIngrd, a homogenous vector of ingredients to be combined
	 * @return a vector of the ingredients, as combined
	 */
	private Vector combine(Vector aIngrd) {
		for(int i = 0; i < aIngrd.size(); i++) {
			for(int j = i+1; j < aIngrd.size(); j++) {
				eShopperIngredient top = (eShopperIngredient) aIngrd.elementAt(i);
				eShopperIngredient bottom = (eShopperIngredient) aIngrd.elementAt(j); 
				if(bottom.canCombineWith(top)) {
					bottom.combineWith(top, largest);
					aIngrd.remove(top);
					i--;  // gotta back up i after a delete
					break;
				}
			}
		}
		return aIngrd;
	}
}
