package eShopper;

import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JComboBox;

/*
 * Created on Apr 29, 2004
 * 
 */

/**
 * All the GUI objects (i.e. windows) inheirit from this class and it contains 
 * the function calls to start any of the other windows, as well as the list 
 * and database attributes they all need.
 * 
 * @author Jim Laing
 */
public abstract class eShopperGUI extends JFrame implements ActionListener  {

	eShopperList list;
	eShopperDB database;
	JComboBox ingredientBox, recipesBox;
	
	/**
	 * This launches the AddItemWindow.
	 *
	 */
	protected void displayAddItemWindow(){
		AddItemWindow anAddItemWindow = new AddItemWindow(list);
		anAddItemWindow.setLocationRelativeTo(this);
		anAddItemWindow.setVisible(true);
	}
	
	/**
	 * This launches the ManageRecipesWindow.
	 *
	 */
	protected void displayManageRecipesWindow(){
		ManageRecipesWindow aManageRecipesWindow = new ManageRecipesWindow(list);
		aManageRecipesWindow.setLocationRelativeTo(this);
		aManageRecipesWindow.setVisible(true);
	}
	
	/**
	 * This launches the ManageIngredientsWindow.
	 *
	 */
	protected void displayManageIngredientsWindow(){
		ManageIngredientsWindow aManageIngredientsWindow = new ManageIngredientsWindow(list);
		aManageIngredientsWindow.setLocationRelativeTo(this);
		aManageIngredientsWindow.setVisible(true);
	}
    
	/**
	 * This launches the CreateRecipeWindow.
	 *
	 */
	protected void displayCreateRecipeWindow(){
		CreateRecipeWindow aCreateRecipeWindow = new CreateRecipeWindow(list, recipesBox);
		aCreateRecipeWindow.setLocationRelativeTo(this);
		aCreateRecipeWindow.setVisible(true);
	}
	
	/**
	 * This launches the CreateIngredientWindow.
	 *
	 */
	protected void displayCreateIngredientWindow(){
		CreateIngredientWindow aCreateIngredientWindow = new CreateIngredientWindow(list, ingredientBox);
		aCreateIngredientWindow.setLocationRelativeTo(this);
		aCreateIngredientWindow.setVisible(true);
	}
	
	
	/** 
	 * This creates an ImageIcon from an image given in the path.
	 * 
	 * @param path The path of the image to be put in the ImageIcon.
	 * @return The ImageIcon, or null if the path was invalid.
	 */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = eShopper.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}
