package eShopper;

import java.sql.*;
import java.util.Vector;
import java.util.zip.DataFormatException;

/*
 * Created on Apr 25, 2004
 * 
 */

/**
 * This class is for the ingredients in a list.
 * It has no children. It is a subclass of eShopperNode.
 * 
 * @author Jim Laing
 */
public class eShopperIngredient extends eShopperNode {
	
	private int ingredientID;
	private int nodeID;
	private float quantity;
	private int unitID;
	
	public eShopperIngredient(int nodeID, int ingredientID,
			int listID, eShopperDB db, float qty, int unitid) {	
		this.nodeID = nodeID;
		this.ingredientID = ingredientID;
		this.listID = listID;
		this.db = db;
		if(!(qty == 0 || unitid == 0))  { 
			this.quantity = qty;
			this.unitID = unitid;
		} else {
			 this.quantity = 1;
			 this.unitID = 0;  // uses default unit
		} 
	}
	
	/**
	 * This returns the children of the Ingredient.
	 * 
	 * @return Always returns null, because an Ingredient cannot have any children.
	 */
	public Vector getChildren() {
		return null;
	}
	
	/**
	 * This overloads the toString() method, and returns the name and
	 * quantity of the Ingredient.
	 */
	public String toString() {
		String qtyStr = new String();
		if(this.quantity == 0 || this.unitID == 0) {
			qtyStr = "";
		} else {
			qtyStr = " -- " + quantity + " " + new eShopperUnit(unitID, db);
		}
		
		try {
			ResultSet rs = db.query("SELECT NAME FROM INGREDIENT WHERE ID = " + ingredientID);
			rs.next();
			return rs.getString("NAME") + qtyStr;
		} catch (SQLException e) {
			System.out.println("Error: eShopperIngredient::toString");
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * This is for getting the name of the Ingredient.
	 * 
	 * @return A string representation of the Ingredient's name.
	 */
	public String getName() {
		try {
			ResultSet rs = db.query("SELECT NAME FROM INGREDIENT WHERE ID = " + ingredientID);
			rs.next();
			return rs.getString("NAME");
		} catch (SQLException e) {
			System.out.println("Error: eShopperIngredient::toString");
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * This accesses the nodeID of the Ingredient
	 * 
	 * @return The nodeID of the Ingredient.
	 */
	public int nodeID(){ return nodeID; }
	
	/**
	 * This accesses the unique ingredientID of the Ingredient.
	 * 
	 * @return The ingredientID.
	 */
	public int ingredientID() { return ingredientID; }
	
	/**
	 * This accesses the description associated with the Ingredient.
	 * 
	 * @return A String representation of the Ingredient's description.
	 */
	public String getDescription() {
		try {
			ResultSet rs = db.query("SELECT DESCRIPTION FROM INGREDIENT WHERE ID = " + ingredientID);
			rs.next();
			return rs.getString("DESCRIPTION");
		} catch (SQLException e) {
			System.out.println("Error: eShopperIngredient::getDescription");
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * This accesses the quantity of the Ingredient in the list.
	 * 
	 * @return The quantity of the Ingredient in the list.
	 */
	public float getQuantity() { return quantity; }
	
	/**
	 * This accesses the UnitID of the Ingredient.
	 * 
	 * @return The UnitID of the Ingredient.
	 */
	public int getUnitID() { return unitID; }
	
	/**
	 * This returns the date associated with the Ingredient.
	 * 
	 * @return A string representation of the date, or null if it's unassigned.
	 */
	public String getDate(){
		String myDate = null;
		// This must be an ingredient of a recipe.  We can't get the date for this.
		if (nodeID == 0)
			return myDate;
		
		ResultSet rs;
		try {
			rs = db.query("SELECT DATE FROM INGREDIENTLIST WHERE ID = " + nodeID);
			if ( rs.next() ){
				myDate = rs.getString("DATE");
			} else {
				myDate = null;
			}
		} catch (SQLException e) {
			System.out.println("Error: eShopperIngredient::date");
			e.printStackTrace();
		}
		return myDate;
	}
	
	/**
	 * This decides whether the quantity of this Ingredient can be combined with
	 * the quantity of another.
	 * 
	 * @param i The Ingredient with which this one should be combined.
	 * @return true if the Ingredients can be combined, false if not.
	 */
	public boolean canCombineWith(eShopperIngredient i) {
		return (new eShopperUnit(i.getUnitID(), db)).isSameType(new eShopperUnit(this.getUnitID(), db));
	}
	
	/**
	 * This combines the quantities of two Ingredients.
	 * 
	 * @param i The Ingredient that this Ingredient should be combined with.
	 * @param largest The largest unit of this type.
	 */
	public void combineWith(eShopperIngredient i, boolean largest) {
		eShopperUnit myUnit = new eShopperUnit(this.unitID, db);
		eShopperUnit iUnit = new eShopperUnit(i.getUnitID(), db);
		try {
			if((largest && iUnit.isLargerThan(myUnit)) || iUnit.isAuth()) {
				// change to iUnit 
				this.quantity = iUnit.express(this.quantity, myUnit);
				this.unitID = iUnit.getID();
				// add amount
				this.quantity += i.getQuantity();
			} else this.quantity += myUnit.express(i.getQuantity(), iUnit); 
			quantity = (float) round(quantity, 2); 
		} catch (DataFormatException e) {
			System.out.println("Error: eShopperIngredient::combineWith");
			e.printStackTrace();
		}
	}
	
	// algorithm taken from 
	// http://www.dickinson.edu/~braught/courses/cs132s01/classes/code/Rounding.src.html
	// thanks to Grant William Braught
	private double round(double val, int places) {
		long factor = (long)Math.pow(10,places);
		val = val * factor;
		long tmp = Math.round(val);
		return (double)tmp / factor;
	}
}