/*
 * Created on May 11, 2004
 *
 */
 
package eShopper;

import java.io.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

/**
 * This class creates a HandyShopper compatible database from the eShopper database.
 * 
 * Takes an eShopperList and creates a handyShopper PDB from it. This can then be
 * put in the Palm install directory using that installTo(path) function
 *
 * Assumes a location of hs2convert in the same directory as eShopper. Also
 * a listSetup.txt file
 * 
 * @author Jim Laing
 *
 */

public class handyShopperDB {
	
	File listCSV = new File("tempList.csv");
	File listMeta = new File("tempList.txt"); 
	File listSetup = new File("listSetup.txt");  // should preexist
	String palmPath;
	eShopperDB db;
	
	/**
	 * Constructs a handy shopper database file from a eShopperList. Attempts
	 * to pass the list to the Palm instapp program whose path is configured 
	 * in the database. Requires hs2convert.exe and a listSetup.txt file which
	 * represents the handyshopper database preferences.
	 * @param list the list which is to be converted
	 */
	public handyShopperDB(eShopperList list) {
		this.db = list.getDB();
		// get the path to the palm install
		try {
			ResultSet rs = db.query("SELECT VALUE FROM CONFIGURE WHERE NAME = 'palm path'");
			rs.next();
			palmPath = rs.getString("VALUE");
		} catch (SQLException e) {
			System.out.println("Error getting palm path: handyShopperDB");
			e.printStackTrace();
		}
		
		try {
			listCSV.createNewFile();
			listMeta.createNewFile();
		} catch (IOException e1) {
			System.out.println("Error creating file: handyShopperDB" );
			e1.printStackTrace();
		}
		
		if(!listCSV.exists() || !listCSV.canWrite() || 
			!listMeta.exists() || !listMeta.canWrite() ||
			!listSetup.exists() || !listMeta.canRead()) {
			System.out.println("File error: handyShopperDB");
		}
		
		else {
			try {
				// first, setup the meta data file
				FileWriter fw = new FileWriter(listMeta);
				FileReader fr = new FileReader(listSetup);
				PrintWriter pw = new PrintWriter(fw);
				pw.println("DatabaseName: " + list.getListName());
				pw.println();
				BufferedReader in = new BufferedReader(fr);
				String line;
				while((line = in.readLine()) != null) pw.println(line);
				fw.close();		
				fr.close();
				
				// create the CSV that will become the database		
				fw = new FileWriter(listCSV);
				pw = new PrintWriter(fw);
				pw.println("Need,Priority,Description,CustomText,Quantity,Units," +
					"Price,Aisle,Date,Category,Stores,PerStoreInfo,EntryOrder,Coupon," +
					"Tax,Tax2,AutoDelete,Private,Note,Alarm,AlarmMidi,Icon");
				Vector v = ((eShopperRoot) ((eShopperTree) list.getModel()).getRoot()).getItemViewVector();
				for(int i=0; i < v.size(); i++) {
					eShopperIngredient ingd = (eShopperIngredient) v.elementAt(i);
					pw.println("x,1," + ingd.getName() + "," + ingd.getDescription() +
						"," + ingd.getQuantity() + "," + new eShopperUnit(ingd.getUnitID(), db) + ",,," 
						+ ",Unfiled,,," + i + ",,,,,,,,0,0");
				}
				fw.close();
				
				// create the handy shopper database
				String cmd[] = { "hs2convert.exe", "-p", "tempList.csv", list.getListName() };
				Process p = Runtime.getRuntime().exec(cmd);
				p.waitFor();
				// send it to instapp
				String cmd2[] = { palmPath + "\\instapp.exe", list.getListName() + ".pdb"};
				p = Runtime.getRuntime().exec(cmd2);
				p.waitFor();
				
				// clean up
				listCSV.delete();
				listMeta.delete();
				new File(list.getListName() + ".pdb").delete();
				
			} catch (Exception e) {
				System.out.println("Error writing HandyShopper file: handyShopperDB");
				e.printStackTrace();
			} 
		}
	}

}
